@@ -17,9 +17,23 @@ limitations under the License.
 package v1alpha2
 
 import (
+	"fmt"
+
 	"k8s.io/apimachinery/pkg/runtime"
 )
 
 func addConversionFuncs(scheme *runtime.Scheme) error {
+	if err := scheme.AddFieldLabelConversionFunc(SchemeGroupVersion.WithKind("ResourceSlice"),
+		func(label, value string) (string, string, error) {
+			switch label {
+			case "metadata.name", "nodeName", "driverName":
+				return label, value, nil
+			default:
+				return "", "", fmt.Errorf("field label not supported for %s: %s", SchemeGroupVersion.WithKind("ResourceSlice"), label)
+			}
+		}); err != nil {
+		return err
+	}
+
 	return nil
 }
