@@ -40,40 +40,6 @@ import (
 	"k8s.io/kubernetes/test/integration/framework"
 )
 
-// TestCustomResourceValidatorsWithDisabledFeatureGate test that x-kubernetes-validations work as expected when the
-// feature gate is disabled.
-func TestCustomResourceValidatorsWithDisabledFeatureGate(t *testing.T) {
-	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, genericfeatures.CustomResourceValidationExpressions, false)()
-
-	server, err := apiservertesting.StartTestServer(t, apiservertesting.NewDefaultTestServerOptions(), nil, framework.SharedEtcd())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer server.TearDownFn()
-	config := server.ClientConfig
-
-	apiExtensionClient, err := clientset.NewForConfig(config)
-	if err != nil {
-		t.Fatal(err)
-	}
-	dynamicClient, err := dynamic.NewForConfig(config)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	t.Run("x-kubernetes-validations fields MUST be dropped from CRDs that are created when feature gate is disabled", func(t *testing.T) {
-		schemaWithFeatureGateOff := crdWithSchema(t, "WithFeatureGateOff", structuralSchemaWithValidators)
-		crdWithFeatureGateOff, err := fixtures.CreateNewV1CustomResourceDefinition(schemaWithFeatureGateOff, apiExtensionClient, dynamicClient)
-		if err != nil {
-			t.Fatal(err)
-		}
-		s := crdWithFeatureGateOff.Spec.Versions[0].Schema.OpenAPIV3Schema
-		if len(s.XValidations) != 0 {
-			t.Errorf("Expected CRD to have no x-kubernetes-validatons rules but got: %v", s.XValidations)
-		}
-	})
-}
-
 // TestCustomResourceValidators tests x-kubernetes-validations compile and validate as expected when the feature gate
 // is enabled.
 func TestCustomResourceValidators(t *testing.T) {
@@ -301,6 +267,13 @@ func TestCustomResourceValidators(t *testing.T) {
 			t.Error("Unexpected error creating custom resource but metadata validation rule")
 		}
 	})
+	t.Run("CRD creation MUST pass for an CRD with empty field", func(t *testing.T) {
+		structuralWithValidators := crdWithSchema(t, "WithEmptyObject", structuralSchemaWithEmptyObject)
+		_, err := fixtures.CreateNewV1CustomResourceDefinition(structuralWithValidators, apiExtensionClient, dynamicClient)
+		if err != nil {
+			t.Errorf("unexpected error creating CRD with empty field: %v", err)
+		}
+	})
 	t.Run("CR creation MUST fail if a x-kubernetes-validations rule exceeds the runtime cost limit", func(t *testing.T) {
 		structuralWithValidators := crdWithSchema(t, "RuntimeCostLimit", structuralSchemaWithCostLimit)
 		crd, err := fixtures.CreateNewV1CustomResourceDefinition(structuralWithValidators, apiExtensionClient, dynamicClient)
@@ -685,103 +658,6 @@ func TestCustomResourceValidatorsWithBlockingErrors(t *testing.T) {
 	})
 }
 
-// TestCustomResourceValidatorsWithSchemaConversion tests CRD replacement with schema conversion issue should not panic.
-func TestCustomResourceValidatorsWithSchemaConversion(t *testing.T) {
-	server, err := apiservertesting.StartTestServer(t, apiservertesting.NewDefaultTestServerOptions(), nil, framework.SharedEtcd())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer server.TearDownFn()
-	config := server.ClientConfig
-
-	apiExtensionClient, err := clientset.NewForConfig(config)
-	if err != nil {
-		t.Fatal(err)
-	}
-	dynamicClient, err := dynamic.NewForConfig(config)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	// Create CRD with normal items+array schema
-	structuralWithValidators := crdWithSchema(t, "Structural", structuralSchemaWithItemsUnderArray)
-	crd, err := fixtures.CreateNewV1CustomResourceDefinition(structuralWithValidators, apiExtensionClient, dynamicClient)
-	if err != nil {
-		t.Fatal(err)
-	}
-	gvr := schema.GroupVersionResource{
-		Group:    crd.Spec.Group,
-		Version:  crd.Spec.Versions[0].Name,
-		Resource: crd.Spec.Names.Plural,
-	}
-	crClient := dynamicClient.Resource(gvr)
-
-	// Create a valid CR instance
-	name1 := names.SimpleNameGenerator.GenerateName("cr-1")
-	_, err = crClient.Create(context.TODO(), &unstructured.Unstructured{Object: map[string]interface{}{
-		"apiVersion": gvr.Group + "/" + gvr.Version,
-		"kind":       crd.Spec.Names.Kind,
-		"metadata": map[string]interface{}{
-			"name": name1,
-		},
-		"spec": map[string]interface{}{
-			"backend": []interface{}{
-				map[string]interface{}{
-					"replicas": 8,
-				},
-			},
-		},
-	}}, metav1.CreateOptions{})
-	if err != nil {
-		t.Errorf("Failed to create custom resource: %v", err)
-	}
-	crd, err = apiExtensionClient.ApiextensionsV1().CustomResourceDefinitions().Get(context.TODO(), crd.Name, metav1.GetOptions{})
-	if err != nil {
-		t.Fatal(err)
-	}
-	structuralSchemaWithItemsUnderObject := crdWithSchema(t, "Structural", structuralSchemaWithItemsUnderObject)
-	structuralSchemaWithItemsUnderObject.SetResourceVersion(crd.GetResourceVersion())
-	// Update CRD with invalid schema items under object
-	crd, err = apiExtensionClient.ApiextensionsV1().CustomResourceDefinitions().Update(context.TODO(), structuralSchemaWithItemsUnderObject, metav1.UpdateOptions{})
-	if err != nil {
-		t.Fatal(err)
-	}
-	// Make an unrelated update to the previous persisted CR instance to make sure CRD handler doesn't panic
-	oldCR, err := crClient.Get(context.TODO(), name1, metav1.GetOptions{})
-	if err != nil {
-		t.Fatal(err)
-	}
-	oldCR.Object["metadata"].(map[string]interface{})["labels"] = map[string]interface{}{"key": "value"}
-	_, err = crClient.Update(context.TODO(), oldCR, metav1.UpdateOptions{})
-	if err == nil || !strings.Contains(err.Error(), "spec.backend in body must be of type object: \"array\", <nil>: Invalid value: \"null\": some validation rules were not checked because the object was invalid; correct the existing errors to complete validation") {
-		t.Fatalf("expect error to contain \"spec.backend in body must be of type object: \"array\", <nil>: Invalid value: \"null\": some validation rules were not checked because the object was invalid; correct the existing errors to complete validation\" but get: %v", err)
-	}
-	// Create another CR instance with an array and be rejected
-	name2 := names.SimpleNameGenerator.GenerateName("cr-2")
-	_, err = crClient.Create(context.TODO(), &unstructured.Unstructured{Object: map[string]interface{}{
-		"apiVersion": gvr.Group + "/" + gvr.Version,
-		"kind":       crd.Spec.Names.Kind,
-		"metadata": map[string]interface{}{
-			"name": name2,
-		},
-		"spec": map[string]interface{}{
-			"backend": []interface{}{
-				map[string]interface{}{
-					"replicas": 7,
-				},
-			},
-		},
-	}}, metav1.CreateOptions{})
-	if err == nil || !strings.Contains(err.Error(), "Invalid value: \"array\": spec.backend in body must be of type object: \"array\"") {
-		t.Fatalf("expect error to contain \"Invalid value: \"array\": spec.backend in body must be of type object: \"array\"\" but get: %v", err)
-	}
-	// Delete the CRD
-	err = fixtures.DeleteV1CustomResourceDefinition(structuralWithValidators, apiExtensionClient)
-	if err != nil {
-		t.Fatal(err)
-	}
-}
-
 func nonStructuralCrdWithValidations() *apiextensionsv1beta1.CustomResourceDefinition {
 	return &apiextensionsv1beta1.CustomResourceDefinition{
 		ObjectMeta: metav1.ObjectMeta{
@@ -1012,76 +888,6 @@ var structuralSchemaWithBlockingErr = []byte(`
   }
 }`)
 
-var structuralSchemaWithItemsUnderArray = []byte(`
-{
-  "openAPIV3Schema": {
-    "description": "CRD with CEL validators",
-    "type": "object",
-    "properties": {
-      "spec": {
-        "type": "object",
-        "properties": {
-          "backend": {
-            "type": "array",
-            "maxItems": 100,
-            "items": {
-              "type": "object",
-              "properties": {
-                "replicas": {
-                  "type": "integer"
-                }
-              },
-              "required": [
-                "replicas"
-              ],
-              "x-kubernetes-validations": [
-                {
-                  "rule": "0 <= self.replicas && self.replicas <= 10"
-                }
-              ]
-            }
-          }
-        }
-      }
-    }
-  }
-}`)
-
-var structuralSchemaWithItemsUnderObject = []byte(`
-{
-  "openAPIV3Schema": {
-    "description": "CRD with CEL validators",
-    "type": "object",
-    "properties": {
-      "spec": {
-        "type": "object",
-        "properties": {
-          "backend": {
-            "type": "object",
-            "maxItems": 100,
-            "items": {
-              "type": "object",
-              "properties": {
-                "replicas": {
-                  "type": "integer"
-                }
-              },
-              "required": [
-                "replicas"
-              ],
-              "x-kubernetes-validations": [
-                {
-                  "rule": "0 <= self.replicas && self.replicas <= 10"
-                }
-              ]
-            }
-          }
-        }
-      }
-    }
-  }
-}`)
-
 var structuralSchemaWithValidMetadataValidators = []byte(`
 {
   "openAPIV3Schema": {
@@ -1268,3 +1074,26 @@ var structuralSchemaWithCostLimit = []byte(`
     }
   }
 }`)
+
+var structuralSchemaWithEmptyObject = []byte(`
+{
+  "openAPIV3Schema": {
+    "description": "weird CRD with empty spec, unstructured status. designed to fit test fixtures.",
+    "type": "object",
+    "x-kubernetes-validations": [
+      {
+        "rule": "[has(self.spec), has(self.status)].exists_one(x, x)"
+      }
+    ],
+    "properties": {
+      "spec": {
+        "type": "object"
+      },
+      "status": {
+        "type": "object",
+        "additionalProperties": true
+      }
+    }
+  }
+}
+`)
